@@ -164,6 +164,14 @@ class GridFile(object):
 
         self.__collection.chunks.remove({"files_id": self.__id})
 
+    @property
+    def _id(self):
+        """The ``"_id"`` value for this :class:`GridFile`.
+
+        .. versionadded:: 1.5.1
+        """
+        return self.__id
+
     @property
     def closed(self):
         """Is this :class:`GridFile` closed?
