@@ -33,17 +33,14 @@
 #include "time_helpers.h"
 #include "encoding_helpers.h"
 
-static PyObject* InvalidName;
-static PyObject* InvalidDocument;
-static PyObject* InvalidStringData;
-static PyObject* SON;
-static PyObject* Binary;
-static PyObject* Code;
-static PyObject* ObjectId;
-static PyObject* DBRef;
-static PyObject* RECompile;
-static PyObject* UUID;
-static PyObject* Timestamp;
+static PyObject* SON = NULL;
+static PyObject* Binary = NULL;
+static PyObject* Code = NULL;
+static PyObject* ObjectId = NULL;
+static PyObject* DBRef = NULL;
+static PyObject* RECompile = NULL;
+static PyObject* UUID = NULL;
+static PyObject* Timestamp = NULL;
 
 #if PY_VERSION_HEX < 0x02050000 && !defined(PY_SSIZE_T_MIN)
 typedef int Py_ssize_t;
@@ -192,12 +189,64 @@ static int write_string(bson_buffer* buffer, PyObject* py_string) {
     return 1;
 }
 
+/* Get an error class from the pymongo.errors module.
+ *
+ * Returns a new ref */
+static PyObject* _error(char* name) {
+    PyObject* error;
+    PyObject* errors = PyImport_ImportModule("pymongo.errors");
+    if (!errors) {
+        return NULL;
+    }
+    error = PyObject_GetAttrString(errors, name);
+    Py_DECREF(errors);
+    return error;
+}
+
+/* Reload a cached Python object.
+ *
+ * Returns non-zero on failure. */
+static int _reload_object(PyObject** object, char* module_name, char* object_name) {
+    PyObject* module;
+
+    module = PyImport_ImportModule(module_name);
+    if (!module) {
+        return 1;
+    }
+
+    *object = PyObject_GetAttrString(module, object_name);
+    Py_DECREF(module);
+
+    return (*object) ? 0 : 2;
+}
+
+/* Reload all cached Python objects.
+ *
+ * Returns non-zero on failure. */
+static int _reload_python_objects(void) {
+    if (_reload_object(&SON, "pymongo.son", "SON") ||
+        _reload_object(&Binary, "pymongo.binary", "Binary") ||
+        _reload_object(&Code, "pymongo.code", "Code") ||
+        _reload_object(&ObjectId, "pymongo.objectid", "ObjectId") ||
+        _reload_object(&DBRef, "pymongo.dbref", "DBRef") ||
+        _reload_object(&Timestamp, "pymongo.timestamp", "Timestamp") ||
+        _reload_object(&RECompile, "re", "compile")) {
+        return 1;
+    }
+    /* If we couldn't import uuid then we must be on 2.4. Just ignore. */
+    if (_reload_object(&UUID, "uuid", "UUID") == 1) {
+        UUID = NULL;
+        PyErr_Clear();
+    }
+    return 0;
+}
+
 /* TODO our platform better be little-endian w/ 4-byte ints! */
 /* Write a single value to the buffer (also write it's type_byte, for which
  * space has already been reserved.
  *
  * returns 0 on failure */
-static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject* value, unsigned char check_keys) {
+static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject* value, unsigned char check_keys, unsigned char first_attempt) {
     /* TODO this isn't quite the same as the Python version:
      * here we check for type equivalence, not isinstance in some
      * places. */
@@ -270,7 +319,7 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
             free(name);
 
             item_value = PySequence_GetItem(value, i);
-            if (!write_element_to_buffer(buffer, list_type_byte, item_value, check_keys)) {
+            if (!write_element_to_buffer(buffer, list_type_byte, item_value, check_keys, 1)) {
                 Py_DECREF(item_value);
                 return 0;
             }
@@ -393,8 +442,10 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         status = check_string((const unsigned char*)PyString_AsString(value),
                               PyString_Size(value), 1, 0);
         if (status == NOT_UTF_8) {
+            PyObject* InvalidStringData = _error("InvalidStringData");
             PyErr_SetString(InvalidStringData,
                             "strings in documents must be valid UTF-8");
+            Py_DECREF(InvalidStringData);
             return 0;
         }
         result = write_string(buffer, value);
@@ -466,7 +517,7 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
     } else if (PyObject_IsInstance(value, Timestamp)) {
         PyObject* obj;
         long i;
-        
+
         obj = PyObject_GetAttrString(value, "inc");
         if (!obj) {
             return 0;
@@ -526,12 +577,16 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         status = check_string((const unsigned char*)PyString_AsString(encoded_pattern),
                               PyString_Size(encoded_pattern), check_utf8, 1);
         if (status == NOT_UTF_8) {
+            PyObject* InvalidStringData = _error("InvalidStringData");
             PyErr_SetString(InvalidStringData,
                             "regex patterns must be valid UTF-8");
+            Py_DECREF(InvalidStringData);
             return 0;
         } else if (status == HAS_NULL) {
+            PyObject* InvalidDocument = _error("InvalidDocument");
             PyErr_SetString(InvalidDocument,
                             "regex patterns must not contain the NULL byte");
+            Py_DECREF(InvalidDocument);
             return 0;
         }
 
@@ -572,78 +627,29 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         }
         *(buffer->buffer + type_byte) = 0x0B;
         return 1;
-    } else {
-        /* Try reloading these modules and see if it works.
-         * This is just to be able to raise a more informative
-         * exception in the weird case that one of these modules was reloaded
-         * without the c extension being reloaded. */
-        PyObject* module;
-
-        module = PyImport_ImportModule("pymongo.binary");
-        if (!module) {
+    } else if (first_attempt) {
+        /* Try reloading the modules and having one more go at it. */
+        if (PyErr_WarnEx(PyExc_RuntimeWarning, "couldn't encode - reloading "
+                         "python modules and trying again. if you see this "
+                         "without getting an InvalidDocument exception "
+                         "please see "
+                         "http://api.mongodb.org/python/current/faq.html#does-pymongo-work-with-mod-wsgi",
+                         1) == -1) {
             return 0;
         }
-        Binary = PyObject_GetAttrString(module, "Binary");
-        Py_DECREF(module);
-
-        module = PyImport_ImportModule("pymongo.code");
-        if (!module) {
-            return 0;
-        }
-        Code = PyObject_GetAttrString(module, "Code");
-        Py_DECREF(module);
-
-        module = PyImport_ImportModule("pymongo.objectid");
-        if (!module) {
-            return 0;
-        }
-        ObjectId = PyObject_GetAttrString(module, "ObjectId");
-        Py_DECREF(module);
-
-        module = PyImport_ImportModule("pymongo.dbref");
-        if (!module) {
-            return 0;
-        }
-        DBRef = PyObject_GetAttrString(module, "DBRef");
-        Py_DECREF(module);
-
-        module = PyImport_ImportModule("pymongo.timestamp");
-        if (!module) {
-            return 0;
-        }
-        Timestamp = PyObject_GetAttrString(module, "Timestamp");
-        Py_DECREF(module);
-
-        module = PyImport_ImportModule("uuid");
-        if (!module) {
-            UUID = NULL;
-            PyErr_Clear();
-        } else {
-            UUID = PyObject_GetAttrString(module, "UUID");
-            Py_DECREF(module);
-        }
-
-        if (PyObject_IsInstance(value, Binary) ||
-            PyObject_IsInstance(value, Code) ||
-            PyObject_IsInstance(value, ObjectId) ||
-            PyObject_IsInstance(value, DBRef) ||
-            PyObject_IsInstance(value, Timestamp) ||
-            (UUID && PyObject_IsInstance(value, UUID))) {
-
-            PyErr_SetString(PyExc_RuntimeError,
-                            "A python module was reloaded without the C extension being reloaded.\n"
-                            "\n"
-                            "See http://api.mongodb.org/python/current/faq.html#does-pymongo-work-with-mod-wsgi for "
-                            "a possible explanation / fix.");
+        if (_reload_python_objects()) {
             return 0;
         }
+        return write_element_to_buffer(buffer, type_byte, value, check_keys, 0);
     }
     {
         PyObject* errmsg = PyString_FromString("Cannot encode object: ");
         PyObject* repr = PyObject_Repr(value);
+        PyObject* InvalidDocument = _error("InvalidDocument");
         PyString_ConcatAndDel(&errmsg, repr);
         PyErr_SetString(InvalidDocument, PyString_AsString(errmsg));
         Py_DECREF(errmsg);
+        Py_DECREF(InvalidDocument);
         return 0;
     }
 }
@@ -652,16 +658,20 @@ static int check_key_name(const char* name,
                           const Py_ssize_t name_length) {
     int i;
     if (name_length > 0 && name[0] == '$') {
+        PyObject* InvalidName = _error("InvalidName");
         PyObject* errmsg = PyString_FromFormat("key '%s' must not start with '$'", name);
         PyErr_SetString(InvalidName, PyString_AsString(errmsg));
         Py_DECREF(errmsg);
+        Py_DECREF(InvalidName);
         return 0;
     }
     for (i = 0; i < name_length; i++) {
         if (name[i] == '.') {
+            PyObject* InvalidName = _error("InvalidName");
             PyObject* errmsg = PyString_FromFormat("key '%s' must not contain '.'", name);
             PyErr_SetString(InvalidName, PyString_AsString(errmsg));
             Py_DECREF(errmsg);
+            Py_DECREF(InvalidName);
             return 0;
         }
     }
@@ -691,7 +701,7 @@ static int write_pair(bson_buffer* buffer, const char* name, Py_ssize_t name_len
     if (!buffer_write_bytes(buffer, name, name_length + 1)) {
         return 0;
     }
-    if (!write_element_to_buffer(buffer, type_byte, value, check_keys)) {
+    if (!write_element_to_buffer(buffer, type_byte, value, check_keys, 1)) {
         return 0;
     }
     return 1;
@@ -711,8 +721,10 @@ static int decode_and_write_pair(bson_buffer* buffer,
                               PyString_Size(encoded), 0, 1);
 
         if (status == HAS_NULL) {
+            PyObject* InvalidDocument = _error("InvalidDocument");
             PyErr_SetString(InvalidDocument,
                             "Key names must not contain the NULL byte");
+            Py_DECREF(InvalidDocument);
             return 0;
         }
     } else if (PyString_Check(key)) {
@@ -724,19 +736,25 @@ static int decode_and_write_pair(bson_buffer* buffer,
                                        PyString_Size(encoded), 1, 1);
 
         if (status == NOT_UTF_8) {
+            PyObject* InvalidStringData = _error("InvalidStringData");
             PyErr_SetString(InvalidStringData,
                             "strings in documents must be valid UTF-8");
+            Py_DECREF(InvalidStringData);
             return 0;
         } else if (status == HAS_NULL) {
+            PyObject* InvalidDocument = _error("InvalidDocument");
             PyErr_SetString(InvalidDocument,
                             "Key names must not contain the NULL byte");
+            Py_DECREF(InvalidDocument);
             return 0;
         }
     } else {
+        PyObject* InvalidDocument = _error("InvalidDocument");
         PyObject* errmsg = PyString_FromString("documents must have only string keys, key was ");
         PyObject* repr = PyObject_Repr(key);
         PyString_ConcatAndDel(&errmsg, repr);
         PyErr_SetString(InvalidDocument, PyString_AsString(errmsg));
+        Py_DECREF(InvalidDocument);
         Py_DECREF(errmsg);
         return 0;
     }
@@ -837,8 +855,10 @@ static int write_dict(bson_buffer* buffer, PyObject* dict, unsigned char check_k
     }
     length = buffer->position - start_position;
     if (length > 4 * 1024 * 1024) {
+        PyObject* InvalidDocument = _error("InvalidDocument");
         PyErr_SetString(InvalidDocument, "document too large - "
                         "BSON documents are limited to 4 MB");
+        Py_DECREF(InvalidDocument);
         return 0;
     }
     memcpy(buffer->buffer + length_location, &length, 4);
@@ -1461,8 +1481,12 @@ static PyObject* get_value(const char* buffer, int* position, int type) {
             break;
         }
     default:
-        PyErr_SetString(InvalidDocument, "no c decoder for this type yet");
-        return NULL;
+        {
+            PyObject* InvalidDocument = _error("InvalidDocument");
+            PyErr_SetString(InvalidDocument, "no c decoder for this type yet");
+            Py_DECREF(InvalidDocument);
+            return NULL;
+        }
     }
     return value;
 }
@@ -1583,7 +1607,6 @@ static PyMethodDef _CBSONMethods[] = {
 
 PyMODINIT_FUNC init_cbson(void) {
     PyObject *m;
-    PyObject* module;
 
     PyDateTime_IMPORT;
     m = Py_InitModule("_cbson", _CBSONMethods);
@@ -1591,70 +1614,6 @@ PyMODINIT_FUNC init_cbson(void) {
         return;
     }
 
-    module = PyImport_ImportModule("pymongo.errors");
-    if (!module) {
-        return;
-    }
-    InvalidName = PyObject_GetAttrString(module, "InvalidName");
-    InvalidDocument = PyObject_GetAttrString(module, "InvalidDocument");
-    InvalidStringData = PyObject_GetAttrString(module, "InvalidStringData");
-    Py_DECREF(module);
-
-    module = PyImport_ImportModule("pymongo.son");
-    if (!module) {
-        return;
-    }
-    SON = PyObject_GetAttrString(module, "SON");
-    Py_DECREF(module);
-
-    module = PyImport_ImportModule("pymongo.binary");
-    if (!module) {
-        return;
-    }
-    Binary = PyObject_GetAttrString(module, "Binary");
-    Py_DECREF(module);
-
-    module = PyImport_ImportModule("pymongo.code");
-    if (!module) {
-        return;
-    }
-    Code = PyObject_GetAttrString(module, "Code");
-    Py_DECREF(module);
-
-    module = PyImport_ImportModule("pymongo.objectid");
-    if (!module) {
-        return;
-    }
-    ObjectId = PyObject_GetAttrString(module, "ObjectId");
-    Py_DECREF(module);
-
-    module = PyImport_ImportModule("pymongo.dbref");
-    if (!module) {
-        return;
-    }
-    DBRef = PyObject_GetAttrString(module, "DBRef");
-    Py_DECREF(module);
-
-    module = PyImport_ImportModule("pymongo.timestamp");
-    if (!module) {
-        return;
-    }
-    Timestamp = PyObject_GetAttrString(module, "Timestamp");
-    Py_DECREF(module);
-
-    module = PyImport_ImportModule("re");
-    if (!module) {
-        return;
-    }
-    RECompile = PyObject_GetAttrString(module, "compile");
-    Py_DECREF(module);
-
-    module = PyImport_ImportModule("uuid");
-    if (!module) {
-        UUID = NULL;
-        PyErr_Clear();
-    } else {
-        UUID = PyObject_GetAttrString(module, "UUID");
-        Py_DECREF(module);
-    }
+    // TODO we don't do any error checking here, should we be?
+    _reload_python_objects();
 }
