@@ -1,4 +1,4 @@
-# Copyright 2009 10gen, Inc.
+# Copyright 2009-2010 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,21 +16,21 @@
 <http://www.mongodb.org/display/DOCS/Object+IDs>`_.
 """
 
-import warnings
 import datetime
-import threading
-import time
-import socket
-import os
-import struct
 try:
     import hashlib
     _md5func = hashlib.md5
-except: # for Python < 2.5
+except ImportError: # for Python < 2.5
     import md5
     _md5func = md5.new
+import os
+import socket
+import struct
+import threading
+import time
+import warnings
 
-from errors import InvalidId
+from pymongo.errors import InvalidId
 
 
 def _machine_bytes():
@@ -68,6 +68,8 @@ class ObjectId(object):
            hexadecimal digits).
 
         .. _ObjectId: http://www.mongodb.org/display/DOCS/Object+IDs
+
+        .. mongodoc:: objectids
         """
         if oid is None:
             self.__generate()
@@ -121,12 +123,13 @@ class ObjectId(object):
             raise TypeError("id must be an instance of (str, ObjectId), "
                             "not %s" % type(oid))
 
+    @property
     def binary(self):
         """12-byte binary representation of this ObjectId.
         """
         return self.__id
-    binary = property(binary)
 
+    @property
     def generation_time(self):
         """A :class:`datetime.datetime` instance representing the time of
         generation for this :class:`ObjectId`.
@@ -138,7 +141,6 @@ class ObjectId(object):
         """
         t = struct.unpack(">i", self.__id[0:4])[0]
         return datetime.datetime.utcfromtimestamp(t)
-    generation_time = property(generation_time)
 
     def __str__(self):
         return self.__id.encode("hex")
